==> lmc-0.6.1.tar.gz: lmc-0.6.1/src/PicturePreview.py <==
# -*- coding: utf-8 -*-
# Author:   $Author: merkosh $
# Revision: $Rev: 105 $
############################################################################
#    Copyright (C) 2005 by Uwe Mayer                                       #
#    merkosh@hadiko.de                                                     #
#                                                                          #
#    This program is free software; you can redistribute it and/or modify  #
#    it under the terms of the GNU General Public License as published by  #
#    the Free Software Foundation; either version 2 of the License, or     #
#    (at your option) any later version.                                   #
#                                                                          #
#    This program is distributed in the hope that it will be useful,       #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of        #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         #
#    GNU General Public License for more details.                          #
#                                                                          #
#    You should have received a copy of the GNU General Public License     #
#    along with this program; if not, write to the                         #
#    Free Software Foundation, Inc.,                                       #
#    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             #
############################################################################

#-- imports --------------------------------------------------------------------
#-- Qt imports
from qt import QImage, QPixmap, QLabel, QGridLayout, PYSIGNAL
from qt import QImageDrag, QDragObject, QSizePolicy


#-- Python imports
import logging
import sys
import os

from os.path import exists as fexists, dirname, splitext, join as fjoin


#-- own imports
from PicturePreviewBase import PicturePreviewBase
from PictureDialog import PictureDialog
from DragObject import XMozUrlDrag, URIListDrag
from misc import openResource
from IODeviceBuffer import IODeviceBuffer
from Settings import getPreferences


#-- constants ------------------------------------------------------------------
# defines maximum number of characters which are recognized as a file suffix
# rather than binary image data
MAX_SUFFIX_LENGTH = 20


#-- preferences ----------------------------------------------------------------
pref = getPreferences(__name__)



class DropLabel(QLabel):
    """label with support for dropping images"""
    def __init__(self, parent=None, name=None, fl=0):
        QLabel.__init__(self, parent, name, fl)
        self.setAcceptDrops(True)
        self.setSizePolicy( QSizePolicy(QSizePolicy.Ignored, QSizePolicy.Ignored) )
        
        # create logging object
        self.log = logging.getLogger( self.__class__.__name__ )
        self.log.setLevel(pref['logLevel'])


    def dragEnterEvent(self, e):
        """called when drop is initiated"""
        accept = QImageDrag.canDecode(e) or XMozUrlDrag.canDecode(e) or \
                 URIListDrag.canDecode(e)
        self.log.debug("Accepting source: %s (type=\"%s\")", str(accept), e.format())
        e.accept( accept )


    def dropEvent(self, e):
        """called when image is dropped on the widget"""
        img = QImage()
        if (QImageDrag.decode(e, img) or XMozUrlDrag.decode(e, img) or \
            URIListDrag.decode(e, img)):
            self.log.debug("Accepting dropped image: %dx%d (%d bytes)",
                           img.width(), img.height(), img.numBytes())
            self.emit(PYSIGNAL("imageDropped"), (img,))
        else:
            self.log.warning("Could not decode dropped image")




class PicturePreview(PicturePreviewBase):
    """displays a picture and scales it appropriately"""
    def __init__(self, dirname, parent=None, name=None, fl=0):
        PicturePreviewBase.__init__(self, parent, name, fl)

        # create logging
        self.log = logging.getLogger( self.__class__.__name__ )
        self.log.setLevel( pref['logLevel'] )

        # runtime variables
        self.basedir = dirname          # remember picture base directory
        self.record = None
        self.img = None                 # original image object
        self.pix = QPixmap()            # pixmap
        self.labelImage = DropLabel(self) # label with drop support

        gridLayoutLabel = QGridLayout( self, 1,1 )
        gridLayoutLabel.addWidget( self.labelImage, 0,0 )

        # connect signal to detect dropped images
        self.connect(self.labelImage, PYSIGNAL("imageDropped"), self.onImageDropped)



    #-- event handlers ---------------------------------------------------------
    def resizeEvent(self, e):
        """called when widget is resized"""
        self.resizeImage()
        

    def mousePressEvent(self, e):
        """called when a mouse button is pressed"""
        # only accept left mouse button
        if (e.button() != e.LeftButton):
            e.ignore()
        # spawn unscaled picture
        if (self.img != None):
            dlg = PictureDialog(self.img, modal=False)
            dlg.exec_loop()
        

    def onImageDropped(self, img):
        """called when image dropped"""
        self.img = img
        self.record.modified(True)      # causes record to be saved
        self.imageDecode()              # update record
        self.resizeImage()              # update on screen


    #-- interface --------------------------------------------------------------
    def resizeImage(self):
        """puts the image in a label on the screen at the correct size"""
        if (self.img != None):
            size = (self.labelImage.contentsRect().width(), \
                    self.labelImage.contentsRect().height())
            img = self.img.smoothScale( size[0], size[1] , QImage.ScaleMin )
            self.pix.convertFromImage(img)
            self.labelImage.setPixmap(self.pix)

        else:
            self.log.debug("No image set: clearing label")
            self.labelImage.clear()


    def getRecord(self):
        """reads the displayed image back into the record and returns it"""
        self.log.debug("Returning record: %d, %s", self.record['number'], self.record['mediaLabel'])

        # create empty data if none set
        if (self.record == None): return None
        # only update picture data if it was modified
        return self.record


    def imageDecode(self):
        """decodes internal image to be saved in the record"""
        #-- handle embedded image
        if (pref['embeddedImage']):
            # extract image from QImage 
            data = IODeviceBuffer()
            if ( self.img.save(data, pref['imageFormat']) ):
                self.log.debug("Saving embedded image: %d bytes", data.size())
                self.record['embeddedImage'] = data.readAll()
                self.record['picture'] = ".%s"%pref['imageFormat'].lower()
                data.close()            # free memory
            else:
                self.log.error("Could not save image to record field")
                self.record['embeddedImage'] = ""
                self.record['picture'] = u""

        #-- handle save image as link
        else:
            pathToPicture = self.basedir # assume relative path
            
            # absolute picture path or linked image
            if ((pref['filenamePrefix'][0] in ('/', )) or
                (pref['filenamePrefix'][:7] in ('http://',))):
                pathToPicture = ""

            # build filename from prefix (including suffix)
            try:
                filename = (pref['filenamePrefix']%self.record['number'])+\
                           ".%s"%pref['imageFormat'].lower()
            except TypeError, e:
                self.log.error("Invalid filename prefix: %s", unicode(e))
                self.record['embeddedImage'] = ""
                self.record['picture'] = u""

            # create target directory
            abs_filename = fjoin(pathToPicture, filename)
            filedir = dirname(abs_filename)
            if (not fexists(filedir)):
                self.log.info("Creating target directory %s", filedir)
                try:
                    os.makedirs(filedir)
                except IOError, e:
                    self.log.error("Could not create target directory: %s", unicode(e))
                    self.record['embeddedImage'] = ""
                    self.record['picture'] = u""

            # test if file exists
            if (fexists(abs_filename)):
                self.log.warning("Target file %s already exists: overwriting", abs_filename)

            # copy data to target file
            data = IODeviceBuffer()
            # copy to target buffer is successfull
            if (self.img == None):
                self.log.info("No picture data to save")
            elif (self.img.save(data, pref['imageFormat'])):
                self.log.debug("Saving linked image %s: %d bytes", abs_filename, data.size())
                try:
                    f = open(abs_filename, "wb")
                    f.write(data.readAll())
                    f.close
                    data.close()        # free memory
                except IOError, e:
                    self.log.error("Could not write data to disk: %s", unicode(e))
                    self.record['embeddedImage'] = ""
                    self.record['picture'] = u""
                else:
                    self.record['embeddedImage'] = str(splitext(filename)[1]) # MUST be ASCII string
                    self.record['picture'] = filename                
            # could not save data to target buffer
            else:
                self.log.error("Could not save image to intermediate buffer")
                self.record['embeddedImage'] = ""
                self.record['picture'] = u""
        
        
    def setRecord(self, rec):
        """sets a record and displays the picture"""
        self.log.debug("Setting record: %d, %s", rec['number'], rec['mediaLabel'])
        # remember record
        self.record = rec

        #-- read record fields and put picture, if available on streen
        if (self.record == None):
            self.img = None
            
        # record has embedded image   
        elif (len(self.record['embeddedImage']) > MAX_SUFFIX_LENGTH):
            self.log.debug("Loading embedded image")
            self.img = QImage()
            ok = self.img.loadFromData(self.record['embeddedImage'])
            if (not ok):
                self.log.error("Could not identify embedded image data")
                self.img = None
                
        # try loading picture from reference
        elif (self.record['picture']):
            self.log.debug("Loading linked image: %s", self.record['picture'])

            pathToPicture = self.basedir # assume relative path
            
            # absolute picture path or linked image
            if ((self.record['picture'][0] in ('/',)) or
                (self.record['picture'][:7] in ('http://',))):
                pathToPicture = ""
                
            # load picture
            f = openResource(fjoin(pathToPicture, self.record['picture']))
            if (f != None):
                self.img = QImage()
                ok = self.img.loadFromData(f.read())
                f.close()
                if (not ok):
                    self.log.error("Could not identify embedded image data")
                    self.img = None
            # could not access file
            else:
                self.img = None
                
        # new record: display no picture
        else:
            self.img = None

        # at this point the image can be:
        # - from http (or whatever openResource loads)
        # - from (correctly) linked image
        # - from temp linked image
        # - embedded data
        # => 2 of 4 reasonst which need explicid decoding
        # image is automatically decoded on drag&drop
        # main program loop must image decode manually when
        # image is loaded from script
        self.resizeImage()




#-- testroutine ----------------------------------------------------------------
if (__name__ == "__main__"):
    from qt import QApplication, QObject, SIGNAL, SLOT
    from AMCFile33 import AMCFile33

    a = QApplication(sys.argv)
    QObject.connect(a,SIGNAL("lastWindowClosed()"),a,SLOT("quit()"))

    # need logging handlers for testing
    log = logging.getLogger("")
    hdlr = logging.StreamHandler()
    formatter = logging.Formatter('[%(name)s] %(levelname)s: %(message)s')
    hdlr.setFormatter(formatter)
    log.addHandler(hdlr) 
    log.setLevel(pref['logLevel'])

    # read sample record
    f = AMCFile33("Filme.amc", "rb")
    rec = f.read()[-1]

    w = PicturePreview(".")
    w.setRecord(rec)

    a.setMainWidget(w)
    w.show()

    a.exec_loop()

    tmp = w.getRecord()
    print "len embedded: ",len(tmp['embeddedImage'])
    print "picture:  ",tmp['picture']

==> lmc-0.6.2.tar.gz: lmc-0.6.2/src/PicturePreview.py <==
# -*- coding: utf-8 -*-
# Author:   $Author: merkosh $
# Revision: $Rev: 108 $
############################################################################
#    Copyright (C) 2005 by Uwe Mayer                                       #
#    merkosh@hadiko.de                                                     #
#                                                                          #
#    This program is free software; you can redistribute it and/or modify  #
#    it under the terms of the GNU General Public License as published by  #
#    the Free Software Foundation; either version 2 of the License, or     #
#    (at your option) any later version.                                   #
#                                                                          #
#    This program is distributed in the hope that it will be useful,       #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of        #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         #
#    GNU General Public License for more details.                          #
#                                                                          #
#    You should have received a copy of the GNU General Public License     #
#    along with this program; if not, write to the                         #
#    Free Software Foundation, Inc.,                                       #
#    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             #
############################################################################

#-- imports --------------------------------------------------------------------
#-- Qt imports
from qt import QImage, QPixmap, QLabel, QGridLayout, PYSIGNAL
from qt import QImageDrag, QDragObject, QSizePolicy, QPopupMenu, QIconSet
from qt import QCursor, QPixmap


#-- Python imports
import logging
import sys
import os

from os.path import exists as fexists, dirname, splitext, join as fjoin


#-- own imports
from PicturePreviewBase import PicturePreviewBase
from PictureDialog import PictureDialog
from DragObject import XMozUrlDrag, URIListDrag
from misc import openResource
from IODeviceBuffer import IODeviceBuffer
from Settings import getPreferences


#-- constants ------------------------------------------------------------------
# defines maximum number of characters which are recognized as a file suffix
# rather than binary image data
MAX_SUFFIX_LENGTH = 20


RECYCLE_BIN = \
    "\x89\x50\x4e\x47\x0d\x0a\x1a\x0a\x00\x00\x00\x0d" \
    "\x49\x48\x44\x52\x00\x00\x00\x10\x00\x00\x00\x10" \
    "\x08\x06\x00\x00\x00\x1f\xf3\xff\x61\x00\x00\x01" \
    "\xa7\x49\x44\x41\x54\x38\x8d\x95\x93\x31\x6b\xdb" \
    "\x40\x18\x86\x1f\x99\x0c\x32\x28\xe0\xa1\x14\x79" \
    "\x8b\x41\x53\x68\x86\x40\x07\x4b\x60\x68\xb6\x72" \
    "\x78\xd3\xd4\x8c\xc9\x96\xa4\x53\x7e\x4a\x43\x36" \
    "\x8f\xce\x2f\x30\x37\x66\x09\xba\xd1\xca\x94\xe9" \
    "\xc0\xa3\x85\xa6\x80\x8d\x3c\x7e\x1d\xcc\xa9\x52" \
    "\xad\x96\x56\x70\x1c\xaf\x78\xf5\xdd\xab\x47\xaf" \
    "\x90\x52\x10\x23\x48\x29\x64\x26\xe3\x7f\x75\x7d" \
    "\xb3\xb9\x03\xf2\xfb\xea\xf2\x49\x29\x78\x52\x0a" \
    "\xc6\x1a\x92\x28\xc1\xfb\xe8\x09\xc0\x42\x2f\x28" \
    "\x5e\x0a\xb4\xd5\xa8\x48\x11\x4e\x42\xa6\x6a\x0a" \
    "\x80\x94\xe2\x39\x3f\x16\xbc\xcc\x64\xf5\xc3\x37" \
    "\xdf\x6f\x28\x8a\x02\x80\xf8\x73\x4c\xea\xa7\xcc" \
    "\xb7\x73\xf2\xd7\x1c\x80\x30\x0c\x79\xfc\xf1\xd8" \
    "\x1a\x72\xe4\x26\xb9\xeb\xfe\xeb\x3d\xf9\x36\xe7" \
    "\x6a\x72\xc5\x73\xf1\xcc\xed\x87\x5b\x9e\x82\x27" \
    "\xce\x83\x73\xe6\xaf\xf3\xbd\xc9\x42\x12\x25\x18" \
    "\x6b\x7e\x31\xe8\x7a\xef\x3f\xad\xbf\x32\x78\x98" \
    "\x3e\xf0\x36\x7a\xe3\x7a\x72\xcd\x8a\x15\x23\x46" \
    "\xcc\x5e\x66\x9c\xae\x4e\xb9\x5b\xdc\x1d\x30\xe8" \
    "\x39\x61\xac\x01\x60\x7d\xb6\x66\x18\x0e\xc9\x37" \
    "\x39\xe9\xa7\x94\x7c\x93\x33\x0c\x87\xac\xcf\xd6" \
    "\x00\xb4\xfc\xd1\x9e\x6a\xfd\x5d\x01\xc9\x66\x99" \
    "\xa8\xa9\x92\x6a\x59\xc9\xe5\xb7\x4b\xa9\x96\x95" \
    "\xa8\xa9\x92\x6c\x96\xd5\xf1\x9b\x3d\xe8\xb9\x49" \
    "\x2e\x81\xb6\x9a\xdd\x76\x87\xb6\x9a\xf7\xcd\x7b" \
    "\x4b\xbb\x04\xce\x9f\x44\x09\xad\x52\xb8\x04\xf1" \
    "\x38\x96\x6a\x59\xc9\xc5\x97\x0b\xa9\x96\x95\xc4" \
    "\xe3\xb8\x95\xa0\xd9\xc8\x03\x06\xda\x6a\x7c\xdf" \
    "\x47\x5b\x4d\x3f\xe8\xb7\x74\x17\x83\x9e\xa3\x99" \
    "\x44\x09\x00\x2a\x52\xf4\x83\x3e\x2a\x52\x0c\x8e" \
    "\x07\x2d\x0d\xd0\xf4\x1b\x6b\xba\x19\x0c\x8e\x07" \
    "\xfb\x93\x03\xbf\xa5\xbb\x18\x1c\xb9\x49\xcd\x04" \
    "\x79\x98\xa3\x22\x45\x1a\xa6\xec\x82\x1d\xc5\xb8" \
    "\xe0\x84\x93\x3a\x41\xb3\x07\x07\x7f\xd7\xbf\x36" \
    "\xd0\xed\x3f\x01\x02\x90\x9f\xc2\x40\xd7\xb4\x8f" \
    "\x00\x00\x00\x00\x49\x45\x4e\x44\xae\x42\x60\x82"


#-- preferences ----------------------------------------------------------------
pref = getPreferences(__name__)



class DropLabel(QLabel):
    """label with support for dropping images"""
    def __init__(self, parent=None, name=None, fl=0):
        QLabel.__init__(self, parent, name, fl)
        self.setAcceptDrops(True)
        self.setSizePolicy( QSizePolicy(QSizePolicy.Ignored, QSizePolicy.Ignored) )
        
        # create logging object
        self.log = logging.getLogger( self.__class__.__name__ )
        self.log.setLevel(pref['logLevel'])


    def dragEnterEvent(self, e):
        """called when drop is initiated"""
        accept = QImageDrag.canDecode(e) or XMozUrlDrag.canDecode(e) or \
                 URIListDrag.canDecode(e)
        self.log.debug("Accepting source: %s (type=\"%s\")", str(accept), e.format())
        e.accept( accept )


    def dropEvent(self, e):
        """called when image is dropped on the widget"""
        img = QImage()
        if (QImageDrag.decode(e, img) or XMozUrlDrag.decode(e, img) or \
            URIListDrag.decode(e, img)):
            self.log.debug("Accepting dropped image: %dx%d (%d bytes)",
                           img.width(), img.height(), img.numBytes())
            self.emit(PYSIGNAL("imageDropped"), (img,))
        else:
            self.log.warning("Could not decode dropped image")




class PicturePreview(PicturePreviewBase):
    """displays a picture and scales it appropriately"""
    def __init__(self, dirname, parent=None, name=None, fl=0):
        PicturePreviewBase.__init__(self, parent, name, fl)

        # create logging
        self.log = logging.getLogger( self.__class__.__name__ )
        self.log.setLevel( pref['logLevel'] )

        # runtime variables
        self.basedir = dirname          # remember picture base directory
        self.record = None
        self.img = None                 # original image object
        self.pix = QPixmap()            # pixmap
        self.labelImage = DropLabel(self) # label with drop support

        self.recycle_bin = QPixmap()
        self.recycle_bin.loadFromData(RECYCLE_BIN, "PNG")
        
        gridLayoutLabel = QGridLayout( self, 1,1 )
        gridLayoutLabel.addWidget( self.labelImage, 0,0 )

        # connect signal to detect dropped images
        self.connect(self.labelImage, PYSIGNAL("imageDropped"), self.onImageDropped)



    #-- event handlers ---------------------------------------------------------
    def resizeEvent(self, e):
        """called when widget is resized"""
        self.resizeImage()
        

    def mousePressEvent(self, e):
        """called when a mouse button is pressed"""
        # only accept left mouse button
        if ((e.button() == e.RightButton) and (self.img != None)):
            menu = QPopupMenu(self)
            menu.insertItem(QIconSet(self.recycle_bin), self.tr("Delete"), self.deletePicture)
            id = menu.exec_loop(QCursor.pos())
            return
            
        # spawn unscaled picture
        if (self.img != None):
            dlg = PictureDialog(self.img, modal=False)
            dlg.exec_loop()


    def deletePicture(self):
        """removes the picture of the current record"""
        self.log.info("Deleting picture from record")
        self.img = None                 # remove picture

        # remove linked image
        filename = fjoin(self.basedir, self.record['picture'])
        if (self.record['picture'] and fexists(filename)):
            try:
                os.unlink(filename)
            except OSError, e:
                self.log.error("Could not remove linked file: %s", unicode(e))

        # decode and display changes
        self.imageDecode()              # decode to empty picture
        self.resizeImage()
        

    def onImageDropped(self, img):
        """called when image dropped"""
        self.img = img
        self.record.modified(True)      # causes record to be saved
        self.imageDecode()              # update record
        self.resizeImage()              # update on screen


    #-- interface --------------------------------------------------------------
    def resizeImage(self):
        """puts the image in a label on the screen at the correct size"""
        if (self.img != None):
            size = (self.labelImage.contentsRect().width(), \
                    self.labelImage.contentsRect().height())
            img = self.img.smoothScale( size[0], size[1] , QImage.ScaleMin )
            self.pix.convertFromImage(img)
            self.labelImage.setPixmap(self.pix)

        else:
            self.log.debug("No image set: clearing label")
            self.labelImage.clear()


    def getRecord(self):
        """reads the displayed image back into the record and returns it"""
        self.log.debug("Returning record: %d, %s", self.record['number'], self.record['mediaLabel'])

        # create empty data if none set
        if (self.record == None): return None
        # only update picture data if it was modified
        return self.record


    def imageDecode(self):
        """decodes internal image to be saved in the record"""
        #-- handle embedded image
        if (pref['embeddedImage']):
            # extract image from QImage 
            data = IODeviceBuffer()
            if ( self.img.save(data, pref['imageFormat']) ):
                self.log.debug("Saving embedded image: %d bytes", data.size())
                self.record['embeddedImage'] = data.readAll()
                self.record['picture'] = ".%s"%pref['imageFormat'].lower()
                data.close()            # free memory
            else:
                self.log.error("Could not save image to record field")
                self.record['embeddedImage'] = ""
                self.record['picture'] = u""

        #-- handle save image as link
        else:
            pathToPicture = self.basedir # assume relative path
            
            # absolute picture path or linked image
            if ((pref['filenamePrefix'][0] in ('/', )) or
                (pref['filenamePrefix'][:7] in ('http://',))):
                pathToPicture = ""

            # build filename from prefix (including suffix)
            try:
                filename = (pref['filenamePrefix']%self.record['number'])+\
                           ".%s"%pref['imageFormat'].lower()
            except TypeError, e:
                self.log.error("Invalid filename prefix: %s", unicode(e))
                self.record['embeddedImage'] = ""
                self.record['picture'] = u""

            # create target directory
            abs_filename = fjoin(pathToPicture, filename)
            filedir = dirname(abs_filename)
            if (not fexists(filedir)):
                self.log.info("Creating target directory %s", filedir)
                try:
                    os.makedirs(filedir)
                except IOError, e:
                    self.log.error("Could not create target directory: %s", unicode(e))
                    self.record['embeddedImage'] = ""
                    self.record['picture'] = u""

            # test if file exists
            if (fexists(abs_filename)):
                self.log.warning("Target file %s already exists: overwriting", abs_filename)

            # copy data to target file
            data = IODeviceBuffer()
            # copy to target buffer is successfull
            if (self.img == None):
                self.log.info("No picture data to save")
            elif (self.img.save(data, pref['imageFormat'])):
                self.log.debug("Saving linked image %s: %d bytes", abs_filename, data.size())
                try:
                    f = open(abs_filename, "wb")
                    f.write(data.readAll())
                    f.close
                    data.close()        # free memory
                except IOError, e:
                    self.log.error("Could not write data to disk: %s", unicode(e))
                    self.record['embeddedImage'] = ""
                    self.record['picture'] = u""
                else:
                    self.record['embeddedImage'] = str(splitext(filename)[1]) # MUST be ASCII string
                    self.record['picture'] = filename                
            # could not save data to target buffer
            else:
                self.log.error("Could not save image to intermediate buffer")
                self.record['embeddedImage'] = ""
                self.record['picture'] = u""
        
        
    def setRecord(self, rec):
        """sets a record and displays the picture"""
        self.log.debug("Setting record: %d, %s", rec['number'], rec['mediaLabel'])
        # remember record
        self.record = rec

        #-- read record fields and put picture, if available on streen
        if (self.record == None):
            self.img = None
            
        # record has embedded image   
        elif (len(self.record['embeddedImage']) > MAX_SUFFIX_LENGTH):
            self.log.debug("Loading embedded image")
            self.img = QImage()
            ok = self.img.loadFromData(self.record['embeddedImage'])
            if (not ok):
                self.log.error("Could not identify embedded image data")
                self.img = None
                
        # try loading picture from reference
        elif (self.record['picture']):
            self.log.debug("Loading linked image: %s", self.record['picture'])

            pathToPicture = self.basedir # assume relative path
            
            # absolute picture path or linked image
            if ((self.record['picture'][0] in ('/',)) or
                (self.record['picture'][:7] in ('http://',))):
                pathToPicture = ""
                
            # load picture
            f = openResource(fjoin(pathToPicture, self.record['picture']))
            if (f != None):
                self.img = QImage()
                ok = self.img.loadFromData(f.read())
                f.close()
                if (not ok):
                    self.log.error("Could not identify embedded image data")
                    self.img = None
            # could not access file
            else:
                self.img = None
                
        # new record: display no picture
        else:
            self.img = None

        # at this point the image can be:
        # - from http (or whatever openResource loads)
        # - from (correctly) linked image
        # - from temp linked image
        # - embedded data
        # => 2 of 4 reasonst which need explicid decoding
        # image is automatically decoded on drag&drop
        # main program loop must image decode manually when
        # image is loaded from script
        self.resizeImage()




#-- testroutine ----------------------------------------------------------------
if (__name__ == "__main__"):
    from qt import QApplication, QObject, SIGNAL, SLOT
    from AMCFile33 import AMCFile33

    a = QApplication(sys.argv)
    QObject.connect(a,SIGNAL("lastWindowClosed()"),a,SLOT("quit()"))

    # need logging handlers for testing
    log = logging.getLogger("")
    hdlr = logging.StreamHandler()
    formatter = logging.Formatter('[%(name)s] %(levelname)s: %(message)s')
    hdlr.setFormatter(formatter)
    log.addHandler(hdlr) 
    log.setLevel(pref['logLevel'])

    # read sample record
    f = AMCFile33(u"/home/merkosh/Projekte/Film- Mp3- Hörspiellisten/Filme/LMC/Filme.amc", "rb")
    rec = f.read()[-1]

    w = PicturePreview(u"/home/merkosh/Projekte/Film- Mp3- Hörspiellisten/Filme/LMC")
    w.setRecord(rec)

    a.setMainWidget(w)
    w.show()

    a.exec_loop()

    tmp = w.getRecord()
    print "len embedded: ",len(tmp['embeddedImage'])
    print "picture:  ",tmp['picture']
